@@ -43,6 +43,26 @@ def rindex(arr, e):
 
 
 def batch_map(proc, items, processes=8):
+    '''
+#usage:     
+#pip install std.algorithm
+import os, std
+from std import http
+os.environ['MYSQL_HOST'] = 'your_host'
+os.environ['MYSQL_DATABASE'] = 'your_database'
+from std import MySQL
+data = ((obj['id'], obj['text']) for obj in MySQL.instance.select('website', training=0, limit=256, dictionary=True))
+id, text = zip(*data)
+
+def proc(text):
+    return http.json_post('http://localhost/your_awesomeapi', dict(text=text, lang='en'))
+
+if __name__ == '__main__':
+    label = std.flatten(std.batch_map(proc, std.batches(text, batch_size=16), processes=4))
+    rowcount = MySQL.instance.executemany('update your_table set label = %s, training = 2 where id = %s', [*zip(label, id)])
+    print('rowcount =', rowcount)    
+    '''
+
     from multiprocessing import Pool
     with Pool(processes=processes) as pool:
         return pool.map(proc, items)    
@@ -667,6 +687,12 @@ def batches(listOfElement, batch_size):
         index += 1
     return batches
 
+def flatten(data):
+    result = []
+    for arr in data:
+        result += arr
+        
+    return result
 
 def is_same(list):
     if isinstance(list, GeneratorType):
