@@ -511,6 +511,30 @@ class KerasModel:
                 device=self.device, 
                 **self.metrics))(generator)
 
+    @std.Timer('predict')
+    def predict(self, inputs, **kwargs):
+        batch_size = kwargs.pop('batch_size', len(inputs))
+        
+        device = self.device
+
+        args = {}
+        if self.is_torch:
+            args['torch'] = True
+        else:
+            args['keras'] = True
+
+        args['sort'] = kwargs.get('sort', False)
+        generator = Sequence(inputs,
+                             batch_size=batch_size,
+                             numpify_x=self.numpify_x,
+                             numpify_y=self.numpify_y, **args)
+        
+        return EpochRunner(
+            StepRunner(
+                model=self.model, 
+                training=False,
+                device=self.device)).predict(generator)
+
     def load_weights(self):
         print('\nloading model from', self.modelFile)
         import h5py
@@ -886,59 +910,8 @@ class LambdaCallback(Callback):
     Create a simple callback on the fly using lambda functions.
     """
 
-    def __init__(
-        self,
-        on_before_accelerator_backend_setup=None,
-        setup=None,
-        on_configure_sharded_model=None,
-        teardown=None,
-        on_init_start=None,
-        on_init_end=None,
-        on_fit_start=None,
-        on_fit_end=None,
-        on_sanity_check_start=None,
-        on_sanity_check_end=None,
-        on_train_batch_start=None,
-        on_train_batch_end=None,
-        on_train_epoch_start=None,
-        on_train_epoch_end=None,
-        on_validation_epoch_start=None,
-        on_validation_epoch_end=None,
-        on_test_epoch_start=None,
-        on_test_epoch_end=None,
-        on_epoch_start=None,
-        on_epoch_end=None,
-        on_batch_start=None,
-        on_validation_batch_start=None,
-        on_validation_batch_end=None,
-        on_test_batch_start=None,
-        on_test_batch_end=None,
-        on_batch_end=None,
-        on_train_start=None,
-        on_train_end=None,
-        on_pretrain_routine_start=None,
-        on_pretrain_routine_end=None,
-        on_validation_start=None,
-        on_validation_end=None,
-        on_test_start=None,
-        on_test_end=None,
-        on_exception=None,
-        on_save_checkpoint=None,
-        on_load_checkpoint=None,
-        on_before_backward=None,
-        on_after_backward=None,
-        on_before_optimizer_step=None,
-        on_before_zero_grad=None,
-        on_predict_start=None,
-        on_predict_end=None,
-        on_predict_batch_start=None,
-        on_predict_batch_end=None,
-        on_predict_epoch_start=None,
-        on_predict_epoch_end=None,
-    ):
-        for k, v in locals().items():
-            if k == "self":
-                continue
+    def __init__(self, **kwargs):
+        for k, v in kwargs.items():
             if v is not None:
                 setattr(self, k, v)
 
@@ -986,7 +959,9 @@ class EpochRunner:
                         if isinstance(data, (list, tuple)):
                             data = [data.to(self.device) for data in data]
                         else:
-                            data.to(self.device)
+                            if data.dtype == torch.int:
+                                data = data.long()
+                            data = data.to(self.device)
                         batch[i] = data
                 else:
                     ...
@@ -1009,7 +984,11 @@ class EpochRunner:
                     continue
             else:
                 with torch.no_grad():
-                    loss_per_batch, loss_items, accuracy_items = self.steprunner(*batch)
+                    args = self.steprunner(*batch)
+                    if self.steprunner.loss:
+                        loss_per_batch, loss_items, accuracy_items = args
+                    else:
+                        return args
                    
             if self.is_torch:
                 total_loss += loss_per_batch.sum().item()
@@ -1019,6 +998,8 @@ class EpochRunner:
                     loss_dict[name] += val.sum().item()
                     
                 for name, val in accuracy_items.items():
+                    if val.dtype == torch.bool:
+                        val = val.float()
                     accuracy_dict[name] += val.sum().item()
             else:
                 total_loss += tf.reduce_sum(loss_per_batch).numpy()
@@ -1068,6 +1049,37 @@ class EpochRunner:
             
         return history
 
+    def predict(self, dataloader):
+        
+        if self.is_torch:
+            import torch
+        else:
+            import tensorflow as tf
+            
+        history = []
+        for current, batch in enumerate(dataloader):
+            if self.device >= 0: 
+                if self.is_torch:
+                    if isinstance(batch, tuple):
+                        batch = [*batch]
+
+                    for i, data in enumerate(batch):
+                        if isinstance(data, (list, tuple)):
+                            data = [data.to(self.device) for data in data]
+                        else:
+                            if data.dtype == torch.int:
+                                data = data.long()
+                            data = data.to(self.device)
+                        batch[i] = data
+                else:
+                    ...
+
+            with torch.no_grad():
+                args = self.steprunner(*batch)
+                history.append(args)
+                   
+        return torch.vstack(history)
+
     @property
     def is_torch(self):
         return self.steprunner.is_torch
@@ -1106,35 +1118,38 @@ class StepRunner:
                 traceback.print_exc()
                 raise e
             
-            if isinstance(self.loss, (list, tuple)):
-                loss_dict = {method_name(loss_fn): loss_fn(y_true, y_pred) for loss_fn, y_true, y_pred in zip(self.loss, y_true, y_pred)}
-            else:
-                loss_dict = {method_name(self.loss): self.loss(y_true, y_pred)}
-    
-            loss = sum(loss_dict.values())
-            # backward()
-            if self.optimizer is not None and self.training:
-                loss_mean = torch.mean(loss)
-            
-                if torch.isnan(loss_mean):
-                    print(loss_mean + 'is nan, stop training')
-                    raise Exception('nan error') 
-            
-                if self.accelerator is None:
-                    loss_mean.backward()
+            if self.loss:
+                if isinstance(self.loss, (list, tuple)):
+                    loss_dict = {method_name(loss_fn): loss_fn(y_true, y_pred) for loss_fn, y_true, y_pred in zip(self.loss, y_true, y_pred)}
                 else:
-                    self.accelerator.backward(loss_mean)
+                    loss_dict = {method_name(self.loss): self.loss(y_true, y_pred)}
     
-                self.optimizer.step()
+                loss = sum(loss_dict.values())
+                # backward()
+                if self.optimizer is not None and self.training:
+                    loss_mean = torch.mean(loss)
                 
-                try:
-                    scheduler = self.optimizer.scheduler
-                    if scheduler is not None:
-                        scheduler.step()# Update learning rate schedule
-                except AttributeError:
-                    ...
+                    if torch.isnan(loss_mean):
+                        print(loss_mean + 'is nan, stop training')
+                        raise Exception('nan error') 
+                
+                    if self.accelerator is None:
+                        loss_mean.backward()
+                    else:
+                        self.accelerator.backward(loss_mean)
+        
+                    self.optimizer.step()
                     
-                self.optimizer.zero_grad()
+                    try:
+                        scheduler = self.optimizer.scheduler
+                        if scheduler is not None:
+                            scheduler.step()# Update learning rate schedule
+                    except AttributeError:
+                        ...
+                        
+                    self.optimizer.zero_grad()
+            else:
+                return y_pred
         else:
             import tensorflow as tf
             with tf.GradientTape() as tape:
@@ -1156,7 +1171,6 @@ class StepRunner:
                     else:
                         ...
             
-        # accuracy
         if isinstance(self.accuracy, dict):
             step_metrics = {name: metric_fn(y_true, y_pred) for name, metric_fn in self.accuracy.items()}
         elif isinstance(self.accuracy, (list, tuple)):
